@@ -2,24 +2,35 @@ from __future__ import annotations
 
 import struct
 from dataclasses import dataclass
+from typing import cast
 from uuid import UUID
 
 from home_assistant_bluetooth import BluetoothServiceInfo
 
 UNPACK_IBEACON = struct.Struct(">HHb").unpack
 
-IBEACON_MFR_ID = 76
 
+APPLE_MFR_ID = 76
+IBEACON_FIRST_BYTE = 0x02
+IBEACON_SECOND_BYTE = 0x15
 
-__version__ = "0.2.0"
+__version__ = "0.4.0"
 
-__all__ = ["parse", "iBeaconAdvertisement"]
+__all__ = [
+    "parse",
+    "calculate_distance_meters",
+    "iBeaconAdvertisement",
+    "APPLE_MFR_ID",
+    "IBEACON_FIRST_BYTE",
+    "IBEACON_SECOND_BYTE",
+]
 
 
 @dataclass
 class iBeaconAdvertisement:
     """A dataclass for iBeacon BLE advertisements."""
 
+    name: str
     uuid: UUID
     major: int
     minor: int
@@ -28,12 +39,13 @@ class iBeaconAdvertisement:
     cypress_humidity: float
     rssi: int
     source: str
+    distance: float
 
 
 def parse(service_info: BluetoothServiceInfo) -> iBeaconAdvertisement | None:
-    if IBEACON_MFR_ID not in service_info.manufacturer_data:
+    if APPLE_MFR_ID not in service_info.manufacturer_data:
         return None
-    data = service_info.manufacturer_data[IBEACON_MFR_ID]
+    data = service_info.manufacturer_data[APPLE_MFR_ID]
     if data[0] != 0x02 or data[1] != 0x15:
         return None
 
@@ -42,6 +54,7 @@ def parse(service_info: BluetoothServiceInfo) -> iBeaconAdvertisement | None:
     (major, minor, power) = UNPACK_IBEACON(data[18:23])
 
     return iBeaconAdvertisement(
+        name=service_info.name,
         uuid=UUID("".join(f"{i:02X}" for i in uuid)),
         major=major,
         minor=minor,
@@ -50,4 +63,16 @@ def parse(service_info: BluetoothServiceInfo) -> iBeaconAdvertisement | None:
         cypress_humidity=125 * ((major & 0xFF) * 256) / 65536 - 6.0,
         rssi=service_info.rssi,
         source=service_info.source,
+        distance=calculate_distance_meters(power, service_info.rssi),
     )
+
+
+def calculate_distance_meters(power: int, rssi: int) -> float:
+    """Calculate the distance in meters between the device and the beacon."""
+    if power >= 0:
+        power = -power
+    if rssi == 0:
+        return -1.0
+    if (ratio := rssi * 1.0 / power) < 1.0:
+        return pow(ratio, 10)
+    return cast(float, 0.89976 * pow(ratio, 7.7095) + 0.111)
